///<reference path="node_modules/typescript/bin/typescript.d.ts" />
///<reference path="node_modules/typescript/bin/typescript_internal.d.ts" />
///<reference path="node_modules/typescript/bin/typescriptServices.d.ts" />
///<reference path="typings/node/node.d.ts" />
///<reference path="typings/q/Q.d.ts" />
///<reference path="typings/loaderUtils/loaderUtils.d.ts" />
///<reference path="typings/objectAssign/objectAssign.d.ts" />
///<reference path="typings/colors/colors.d.ts" />
var typescript = require('typescript');
var path = require('path');
var fs = require('fs');
var os = require('os');
var Q = require('q');
var loaderUtils = require('loader-utils');
var objectAssign = require('object-assign');
require('colors');
var readFile = Q.denodeify(fs.readFile);
var instances = {};
function ensureTypeScriptInstance(options) {
    if (Object.prototype.hasOwnProperty.call(instances, options.instance))
        return instances[options.instance];
    var target;
    switch (options.target) {
        case "ES3":
            target = 0 /* ES3 */;
            break;
        case "ES6":
            target = 2 /* ES6 */;
            break;
        default: target = 1 /* ES5 */;
    }
    var compilerOptions = {
        target: target,
        module: options.module == "AMD" ? 2 /* AMD */ : 1 /* CommonJS */,
        sourceMap: !!options.sourceMap
    };
    var files = {};
    var libPath = path.join(path.dirname(require.resolve('typescript')), 'lib.d.ts');
    files[libPath] = {
        text: fs.readFileSync(libPath, 'utf-8'),
        version: 0
    };
    var servicesHost = {
        getScriptFileNames: function () { return Object.keys(files); },
        getScriptVersion: function (filename) { return files[filename] && files[filename].version.toString(); },
        getScriptSnapshot: function (filename) {
            var file = files[filename];
            return {
                getText: function (start, end) { return file.text.substring(start, end); },
                getLength: function () { return file.text.length; },
                getLineStartPositions: function () { return []; },
                getChangeRange: function (oldSnapshot) { return undefined; }
            };
        },
        getCurrentDirectory: function () { return process.cwd(); },
        getScriptIsOpen: function () { return true; },
        getCompilationSettings: function () { return compilerOptions; },
        getDefaultLibFilename: function (options) { return 'lib.d.ts'; },
        // getNewLine() should work in next version of TypeScript
        // see https://github.com/Microsoft/TypeScript/issues/1653
        //getNewLine: () => { return os.EOL },
        log: function (message) { return console.log(message); }
    };
    var languageService = typescript.createLanguageService(servicesHost, typescript.createDocumentRegistry());
    return instances[options.instance] = {
        files: files,
        languageService: languageService
    };
}
function rootReferencePath(referencePath, dirname) {
    return typescript.isRootedDiskPath(referencePath) ? referencePath : typescript.combinePaths(dirname, referencePath);
}
function ensureDependencies(resolver, instance, filePath, contents) {
    filePath = path.normalize(filePath);
    if (!Object.prototype.hasOwnProperty.call(instance.files, filePath)) {
        var fileInfo = typescript.preProcessFile(contents);
        var dirname = path.dirname(filePath);
        var dependencies = fileInfo.referencedFiles.concat(fileInfo.importedFiles).map(function (f) { return ({
            original: f.filename,
            resolved: '',
            pos: f.pos,
            end: f.end,
            reference: fileInfo.referencedFiles.indexOf(f) != -1
        }); });
        instance.files[filePath] = { version: 0, text: contents };
        return Q.all(dependencies.map(function (f) { return resolver(dirname, f.reference ? rootReferencePath(f.original, dirname) : f.original).then(function (newPath) { return f.resolved = newPath; }).then(function (newPath) { return f; }); })).then(function (filePaths) { return filePaths.filter(function (f) { return f.resolved.match(/\.ts$/) != null; }); }).then(function (filePaths) { return Q.all(filePaths.map(function (f) { return readFile(f.resolved, 'utf-8').then(function (fileContents) { return ensureDependencies(resolver, instance, f.resolved, fileContents); }); })); }).then(function () { return contents; });
    }
    return Q(contents);
}
function loader(contents) {
    var _this = this;
    this.cacheable && this.cacheable();
    var callback = this.async();
    var filePath = path.normalize(this.resourcePath);
    var options = loaderUtils.parseQuery(this.query);
    options = objectAssign({}, {
        instance: 'default',
        sourceMap: false
    }, options);
    var instance = ensureTypeScriptInstance(options);
    var resolver = Q.denodeify(this.resolve.bind(this));
    ensureDependencies(resolver, instance, filePath, contents).then(function (contents) {
        var file = instance.files[filePath], langService = instance.languageService;
        file.text = contents;
        file.version++;
        var output = langService.getEmitOutput(filePath);
        var diagnostics = langService.getCompilerOptionsDiagnostics().concat(langService.getSyntacticDiagnostics(filePath)).concat(langService.getSemanticDiagnostics(filePath)).forEach(function (diagnostic) {
            if (diagnostic.file) {
                var lineChar = diagnostic.file.getLineAndCharacterFromPosition(diagnostic.start);
                _this.emitError("  " + diagnostic.file.filename.blue + " (" + lineChar.line.toString().cyan + "," + lineChar.character.toString().cyan + "): " + diagnostic.messageText.red);
            }
            else {
                _this.emitError("  " + "unknown file".blue + ": " + diagnostic.messageText.red);
            }
        });
        if (output.outputFiles.length == 0)
            throw new Error("Typescript emitted no output for " + filePath);
        var sourceMap;
        if (options.sourceMap) {
            sourceMap = JSON.parse(output.outputFiles[0].text);
            sourceMap.sourcesContent = [contents];
            contents = output.outputFiles[1].text;
        }
        else {
            contents = output.outputFiles[0].text;
        }
        contents = contents.replace(/\r\n/g, os.EOL);
        return [contents, sourceMap];
    }).done(function (contents) { return callback(null, contents[0], contents[1]); }, function (err) { return callback(err); });
}
module.exports = loader;
